;; redline/theme.jl

;; based on minimal by Garcia Galego <rgarcia@eurielec.etsit.upm.es>
;; hacked up a bit (m3) by Jacob Berkman <jacob@helixcode.com>
;; hacked up some more by Jonas Linde <jonas@init.se>

;; last update 2000-03-21 -- added customization

;; (custom-set-frame-style 'default-frame-style 'redline)

(defgroup redline "redline theme"
  :group appearance)

(defcustom redline:titlefont "-misc-fixed-medium-r-normal-*-*-100-*-*-c-*-iso8859-1"
  "Title font"
  :group (appearance redline)
  :type font)

(defcustom redline:active-foreground "white"
  "Title foreground color of active window"
  :group (appearance redline)
  :type color
  :after-set after-setting-frame-option)

(defcustom redline:active-background "black"
  "Title background color of active window"
  :group (appearance redline)
  :type color
  :after-set after-setting-frame-option)

(defcustom redline:active-high "red"
  "Sunny side of active window"
  :group (appearance redline)
  :type color
  :after-set after-setting-frame-option)

(defcustom redline:active-low "red4"
  "Shadow side of active window"
  :group (appearance redline)
  :type color
  :after-set after-setting-frame-option)

(defcustom redline:inactive-foreground "grey50"
  "Title foreground color of inactive window"
  :group (appearance redline)
  :type color
  :after-set after-setting-frame-option)

(defcustom redline:inactive-background "black"
  "Title background color of inactive window"
  :group (appearance redline)
  :type color
  :after-set after-setting-frame-option)

(defcustom redline:inactive-high "grey25"
  "Sunny side of inactive window"
  :group (appearance redline)
  :type color
  :after-set after-setting-frame-option)

(defcustom redline:inactive-low "grey25"
  "Shadow side of inactive window"
  :group (appearance redline)
  :type color
  :after-set after-setting-frame-option)

(defcustom redline:border-width 1
  "Border width"
  :group (appearance redline)
  :type number
  :after-set after-setting-frame-option)

(let* (
       
       (title-width
	(lambda (w)
	  (let
	      ((w-width (car (window-dimensions w))))
	    (max 0 (text-width (window-name w))))))
       
       (update-text-width
	(lambda (w)
	  (if (eq (window-get w 'current-frame-style) 'redline)
	      (rebuild-frame w))))
       
       (font (lambda () redline:titlefont))

       (font-colors (lambda () (list redline:inactive-foreground redline:active-foreground)))
       (colors      (lambda () (list redline:inactive-background redline:active-background)))
       (pre-colors  (lambda () (list redline:inactive-high redline:active-high)))
       (post-colors (lambda () (list redline:inactive-low  redline:active-low)))

       (border-width  (lambda() redline:border-width))
       (-border-width (lambda() (- redline:border-width)))

(frame
 `(
   ;; Bar
   ((background . ,colors)
    (foreground . ,font-colors)
    (font . ,font)
    (text . ,window-name)
    (top-edge . 0)
    (height . 11)
    (right-edge . 0)
    (width . ,(lambda (w) (+ (title-width w) 12)))
    (x-justify . center)
    (y-justify . center)
    (class . title))

   ;; Window borders
   ((background . ,pre-colors)
    (top-edge . ,-border-width)
    (right-edge . ,-border-width)
    (left-edge . 0)
    (height . ,border-width)
    (class . title)
    )

   ((background . ,pre-colors)
    (top-edge . ,-border-width)
    (bottom-edge . ,-border-width)
    (left-edge . ,-border-width)
    (width . ,border-width)
    (class . title)
    )

   ((background . ,post-colors)
    (bottom-edge . ,-border-width)
    (left-edge . 0)
    (right-edge . ,-border-width)
    (height . ,border-width)
    (class . title)
    )

   ((background . ,post-colors)
    (top-edge . 0)
    (bottom-edge . 0)
    (right-edge . ,-border-width)
    (width . ,border-width)
    (class . title)
    )

   ;; Bar frame
   ((background . ,post-colors)
    (top-edge . 11)
    (right-edge . 0)
    (height . 1)
    (width . ,(lambda (w) (+ (title-width w) 12)))
    (class . title)
    )

   ((background . ,post-colors)
    (top-edge . -1)
    (right-edge . -1)
    (width . 1)
    (height . 13)
    (class . title)
    )

   ((background . ,pre-colors)
    (top-edge . -1)
    (right-edge . -1)
    (height . 1)
    (width . ,(lambda (w) (+ (title-width w) 14)))
    (class . title)
    )

   ((background . ,pre-colors)
    (top-edge . -1)
    (right-edge . ,(lambda (w) (+ (title-width w) 12)))
    (width . 1)
    (height . 13)
    (class . title)
    )
   )
 )

(shaped-frame
 `(
   ;; Bar
   ((background . ,colors)
    (foreground . ,font-colors)
    (font . ,font)
    (text . ,window-name)
    (top-edge . -12)
    (height . 11)
    (right-edge . 1)
    (width . ,(lambda (w) (+ (title-width w) 12)))
    (x-justify . center)
    (y-justify . center)
    (class . title))

   ;; Bar frame
   ((background . ,post-colors)
    (top-edge . -1)
    (right-edge . 1)
    (height . 1)
    (width . ,(lambda (w) (+ (title-width w) 12)))
    (class . title)
    )

   ((background . ,post-colors)
    (top-edge . -13)
    (right-edge . 0)
    (width . 1)
    (height . 13)
    (class . title)
    )

   ((background . ,pre-colors)
    (top-edge . -13)
    (right-edge . 0)
    (height . 1)
    (width . ,(lambda (w) (+ (title-width w) 14)))
    (class . title)
    )

   ((background . ,pre-colors)
    (top-edge . -13)
    (right-edge . ,(lambda (w) (+ (title-width w) 13)))
    (width . 1)
    (height . 13)
    (class . title)
    )
   )
 )

(transient-frame
 `(
   ;; Window borders
   ((background . ,pre-colors)
    (top-edge . ,-border-width)
    (right-edge . ,-border-width)
    (left-edge . 0)
    (height . ,border-width)
    (class . title)
    )

   ((background . ,pre-colors)
    (top-edge . ,-border-width)
    (bottom-edge . ,-border-width)
    (left-edge . ,-border-width)
    (width . ,border-width)
    (class . title)
    )

   ((background . ,post-colors)
    (bottom-edge . ,-border-width)
    (left-edge . 0)
    (right-edge . ,-border-width)
    (height . ,border-width)
    (class . title)
    )

   ((background . ,post-colors)
    (top-edge . 0)
    (bottom-edge . 0)
    (right-edge . ,-border-width)
    (width . ,border-width)
    (class . title)
    )
   )
 )

)


(add-frame-style 'redline
		 (lambda (w type)
		   (case type
			 ((default) frame)
			 ((shaped) shaped-frame)
			 ((transient) transient-frame)
			 ((shaped-transient) shaped-frame)
			 ((unframed) nil-frame)
			 )))

(call-after-property-changed 'WM_NAME update-text-width)
)
